/*
 * Java
 *
 * Copyright 2019-2021 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.microej.app.wadapps.commandserver.socket;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.microej.wadapps.rcommand.WadappsCommands;

import android.net.ConnectivityManager;
import android.net.ConnectivityManager.NetworkCallback;
import android.net.Network;
import android.net.NetworkInfo;
import ej.components.dependencyinjection.ServiceLoaderFactory;
import ej.net.PollerConnectivityManager;
import ej.rcommand.RemoteCommandManager;
import ej.rcommand.serversocket.RemoteCommandServer;
import ej.wadapps.app.BackgroundService;
import ej.wadapps.registry.SharedRegistryFactory;

/**
 * Service providing {@link RemoteCommandManager} and
 * {@link ej.rcommand.connectivity.ConnectivityManager}.
 */
public class AdminService extends NetworkCallback implements BackgroundService {
	private static final int COMMAND_SERVER_NB_MAX_CONNECTION = 2;
	private static final int COMMAND_SERVER_PORT = 4000;
	private static final String NETWORK_CONNECTIVITY = "Network connectivity "; //$NON-NLS-1$
	private static final String ADMIN_SERVER = "admin server"; //$NON-NLS-1$
	private final RemoteCommandServer adminServer;
	private final Logger logger;
	private final RemoteCommandManager commandManager;

	/**
	 * Instantiates an {@link AdminService}.
	 */
	public AdminService() {
		this.logger = Logger.getLogger(AdminService.class.getSimpleName());
		this.commandManager = ServiceLoaderFactory.getServiceLoader().getService(RemoteCommandManager.class);
		if (this.commandManager == null) {
			throw new RuntimeException("Cannot instanciate an instance of the " + ADMIN_SERVER); //$NON-NLS-1$
		} else {
			WadappsCommands.registerCommands(this.commandManager);
		}
		this.adminServer = new RemoteCommandServer(this.commandManager, COMMAND_SERVER_PORT,
				COMMAND_SERVER_NB_MAX_CONNECTION);
	}

	@Override
	public void onStart() {
		SharedRegistryFactory.getSharedRegistry().register(ej.rcommand.connectivity.ConnectivityManager.class,
				this.adminServer);
		ConnectivityManager connectivityManager = ServiceLoaderFactory.getServiceLoader()
				.getService(ConnectivityManager.class, PollerConnectivityManager.class);
		connectivityManager.registerDefaultNetworkCallback(this);
		NetworkInfo info = connectivityManager.getActiveNetworkInfo();
		if (info.isConnected()) {
			startServer();
		}
	}

	@Override
	public void onStop() {
		this.logger.info("Stop the socket " + ADMIN_SERVER); //$NON-NLS-1$
		stopServer();
		SharedRegistryFactory.getSharedRegistry().unregister(ej.rcommand.connectivity.ConnectivityManager.class,
				this.adminServer);
	}

	@Override
	public void onAvailable(Network network) {
		this.logger.info(NETWORK_CONNECTIVITY + "is now available"); //$NON-NLS-1$
		startServer();
	}

	@Override
	public void onLost(Network network) {
		this.logger.info(NETWORK_CONNECTIVITY + "has been lost"); //$NON-NLS-1$
		stopServer();
	}

	private synchronized void startServer() {
		this.logger.info("Start the socket " + ADMIN_SERVER); //$NON-NLS-1$
		new Thread(this.adminServer, ADMIN_SERVER).start();
	}

	private synchronized void stopServer() {
		this.commandManager.stopAll();
		try {
			this.adminServer.stopRunning();
		} catch (IOException e) {
			this.logger.log(Level.INFO, ADMIN_SERVER, e);
		}
	}
}
