/*
 * Java
 *
 * Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 * For demonstration purpose only.
 * MicroEJ Corp. PROPRIETARY. Use is subject to license terms.
 */
package com.microej.demo.thermorest;

import java.io.IOException;
import java.util.Properties;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.microej.demo.thermorest.cloud.PeriodicPublisherCloud;
import com.microej.demo.thermorest.cloud.aws.CertificateLoader;
import com.microej.demo.thermorest.cloud.aws.ResourceCertificateLoader;
import com.microej.demo.thermorest.cloud.aws.ThermostatAWS;
import com.microej.demo.thermorest.cloud.mqtt.ThermostatMQTT;
import com.microej.demo.thermorest.endpoint.AWSUpload;
import com.microej.demo.thermorest.model.ThermostatListener;

import ej.basictool.ArrayTools;
import ej.net.util.NetUtil;
import ej.net.util.NtpUtil;

/**
 * Entry point of the demo, adding a default cloud connection then starting the server.
 */
public class Main {

	private static final int IP_POLLING_RATE = 1_000;
	private static final Logger LOGGER = Logger.getLogger("DEMO"); //$NON-NLS-1$
	private static final int PORT = 80;
	private static final String MQTT_ID = "MQTT_THERMOREST_TEST"; //$NON-NLS-1$
	private static final String MQTT_BROKER = "tcp://iot.eclipse.org:1883"; //$NON-NLS-1$
	/**
	 * Threshold time to know if local time has been updated.
	 */
	private static final long NTP_MINIMUM_TIME = 1000000;

	/**
	 * Entry Point.
	 *
	 * @param args
	 *            not used.
	 */
	public static void main(String[] args) {
		LOGGER.log(Level.INFO, "Starting demo."); //$NON-NLS-1$
		initNetwork();

		HeadlessThermostat thermostat = new HeadlessThermostat();
		addCloudConnector(thermostat, new Properties(), new ResourceCertificateLoader());
		start(thermostat, PORT);
	}

	/**
	 * Wait for the network, set the current time using NTP.
	 */
	public static void initNetwork() {
		while (NetUtil.getFirstHostAddress() == null) {
			LOGGER.log(Level.INFO, "Waiting for IP."); //$NON-NLS-1$
			try {
				Thread.sleep(IP_POLLING_RATE);
			} catch (InterruptedException e) {
				LOGGER.log(Level.INFO, e.getMessage(), e);
			}
		}
		LOGGER.log(Level.INFO, "Updating time using NTP."); //$NON-NLS-1$
		while (System.currentTimeMillis() < NTP_MINIMUM_TIME) {
			try {
				NtpUtil.updateLocalTime();
			} catch (IOException e) {
				LOGGER.log(Level.INFO, e.getMessage(), e);
			}
		}
	}

	/**
	 * Add cloud connector, AWS if all the properties has been set, MQTT otherwise.
	 *
	 * @param thermostat
	 *            the thermostat.
	 * @param properties
	 *            the properties to use for AWS loading.
	 * @param certificateLoader
	 *            the {@link CertificateLoader} to use.
	 */
	public static void addCloudConnector(HeadlessThermostat thermostat, Properties properties,
			CertificateLoader certificateLoader) {
		PeriodicPublisherCloud connector = createAWSConnector(properties, certificateLoader);
		if (connector == null) {
			connector = new ThermostatMQTT(MQTT_ID + new Random().nextInt(), MQTT_BROKER);
		}
		thermostat.setEndpoints(
				ArrayTools.add(thermostat.getEndpoints(), new AWSUpload(thermostat, certificateLoader, properties)));
		connector.setThermostat(thermostat);
		connector.start();
	}

	/**
	 * Creates an AWS connector.
	 *
	 * @param properties
	 *            the properties to use.
	 * @param certificateLoader
	 *            the certificate loader to use.
	 * @return the AWS connector, <code>null</code> if not available.
	 */
	public static PeriodicPublisherCloud createAWSConnector(Properties properties,
			CertificateLoader certificateLoader) {
		PeriodicPublisherCloud connector = null;
		try {
			connector = new ThermostatAWS(properties, certificateLoader);
		} catch (IOException e) {
			// Should not occur as the properties had been checked before.
			LOGGER.log(Level.INFO, "Could not start AWS", e); //$NON-NLS-1$
		}
		return connector;
	}

	/**
	 * Starts the thermostat.
	 *
	 * @param thermostat
	 *            the thermostat.
	 * @param port
	 *            port of the thermostat.
	 */
	public static void start(HeadlessThermostat thermostat, int port) {
		thermostat.getModel().addListener(new ThermostatListener() {

			@Override
			public void onTemperatureChange(float newTemperature) {
				LOGGER.log(Level.INFO, "Temperature=" + newTemperature); //$NON-NLS-1$
			}

			@Override
			public void onTargetChange(float newTarget) {
				LOGGER.log(Level.INFO, "Target=" + newTarget); //$NON-NLS-1$
			}
		});
		thermostat.setPort(port);
		try {
			thermostat.start();
			LOGGER.log(Level.INFO, "Please browse to http:/" + NetUtil.getFirstHostAddress() + ':' + port); //$NON-NLS-1$
		} catch (IOException e) {
			LOGGER.log(Level.INFO, e.getMessage(), e);
		}
	}
}
