/*
 * Java
 *
 * Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.profiling.internal.snapshot;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.microej.profiling.Group;
import com.microej.profiling.internal.Strings;
import com.microej.profiling.internal.snapshot.Snapshot.Type;
import com.microej.profiling.profiler.ComparableProfiler;
import com.microej.profiling.profiler.HeapProfiler;

import ej.basictool.ArrayTools;
import ej.bon.IllegalStateException;

/**
 * Gather the profiler snapshots.
 */
public class ProfilerSnapshots {

	private static long SNAPSHOT_SIZE;
	private final ComparableProfiler profiler;
	private Snapshot[] snapshots;

	static {
		// Compute the size of a snapshot.
		Runtime runtime = Runtime.getRuntime();
		Snapshot[] snapshots = new Snapshot[0];
		HeapProfiler heapProfiler = new HeapProfiler();
		snapshots = ArrayTools.add(snapshots, new Snapshot(1, null, null, null, heapProfiler));
		System.gc();
		long memory = runtime.freeMemory();
		snapshots = ArrayTools.add(snapshots, new Snapshot(0, null, null, null, heapProfiler));
		System.gc();
		SNAPSHOT_SIZE = memory - runtime.freeMemory();
	}

	/**
	 * Instantiates a {@link PrintableSnapshot}.
	 *
	 * @param profiler
	 *            the profiler to follow.
	 */
	public ProfilerSnapshots(ComparableProfiler profiler) {
		this.profiler = profiler;
		this.snapshots = new Snapshot[0];
	}

	/**
	 * Takes a snapshot.
	 *
	 * @param level
	 *            the stack's level.
	 * @param tag
	 *            the tag.
	 * @param type
	 *            the type.
	 * @param group
	 *            the group.
	 */
	public void snapshot(int level, String tag, Type type, Group group) {
		HeapProfiler.OFFSET += SNAPSHOT_SIZE;
		this.snapshots = ArrayTools.add(this.snapshots, new Snapshot(level, tag, type, group, this.profiler));
	}

	@Override
	public String toString() {
		StringBuilder out = new StringBuilder();
		out.append(Strings.MINUS + this.profiler.getName() + Strings.COLLON + Strings.LINE_BREAK);
		if (this.snapshots.length > 0) {
			Map<Integer, PrintablePair> levels = new HashMap<>();
			List<PrintableSnapshot> printable = new ArrayList<>();
			try {
				for (Snapshot snapshot : this.snapshots) {
					handleSnapshot(levels, printable, snapshot);
				}
			} catch (IllegalStateException e) {
				// Stops the trace.
			}
			List<Integer> openLevels = new ArrayList<>();
			for (int i = 0; i < printable.size(); i++) {
				PrintableSnapshot printableTag = printable.get(i);
				appendTree(out, printableTag, printable, i, openLevels);
				out.append(printableTag + Strings.LINE_BREAK);
			}
		} else {
			out.append(Strings.NO_INFO + Strings.LINE_BREAK);
		}
		return out.toString();
	}

	private void appendTree(StringBuilder out, PrintableSnapshot printableTag, List<PrintableSnapshot> printable, int i,
			List<Integer> openLevels) {
		int level = printableTag.getLevel();
		Iterator<Integer> iterator = openLevels.iterator();
		Integer currentLevel = Integer.valueOf(-1);
		if (iterator.hasNext()) {
			currentLevel = iterator.next();
		}
		for (int j = 0; j < level; j++) {
			if (currentLevel.equals(Integer.valueOf(j))) {
				out.append(Strings.TREE_CARRY_ON_SYMBOL);
				if (iterator.hasNext()) {
					currentLevel = iterator.next();
				}
			} else {
				out.append(' ');
			}
			out.append(Strings.LOWLIGHT_SYMBOL);
			out.append(Strings.TAB);
		}
		if (isEndLeaf(printable, i, level)) {
			out.append(Strings.END_LEAF_SYMBOL);
			openLevels.remove(Integer.valueOf(level));
		} else {
			if (printableTag.getType() == Type.LAP) {
				out.append(Strings.LAP_SYMBOL);
			} else {
				openLevels.add(Integer.valueOf(level));
				out.append(Strings.TREE_CARRY_ON_SYMBOL);
			}
		}
		out.append(Strings.HIGHLIGHT_SYMBOL);
	}

	private boolean isEndLeaf(List<PrintableSnapshot> printable, int i, int level) {
		boolean isLeaf = true;
		for (int j = i + 1; j < printable.size(); j++) {
			int nextLevel = printable.get(j).getLevel();
			if (nextLevel == level) {
				isLeaf = false;
				break;
			} else if (nextLevel < level) {
				isLeaf = true;
				break;
			}
		}
		return isLeaf;
	}

	private void handleSnapshot(Map<Integer, PrintablePair> levels, List<PrintableSnapshot> printable,
			Snapshot snapshot) throws IllegalStateException {
		Integer level = Integer.valueOf(snapshot.getLevel());
		PrintablePair enterSnapshot = levels.get(level);
		switch (snapshot.getType()) {
		case ENTER:
			enter(levels, printable, snapshot, level, enterSnapshot);
			break;
		case EXIT:
			exit(levels, snapshot, level, enterSnapshot);
			break;
		case LAP:
		default:
			tag(levels, printable, snapshot, level, enterSnapshot);
			break;
		}
	}

	private void tag(Map<Integer, PrintablePair> levels, List<PrintableSnapshot> printable, Snapshot snapshot,
			Integer level, PrintablePair enterSnapshot) {
		if (enterSnapshot == null) {
			enter(levels, printable, snapshot, level, enterSnapshot);
		} else {
			PrintableSnapshot last = enterSnapshot.getSecond();
			Snapshot lastTag = last.getEnd();
			if (lastTag == null) {
				lastTag = last.getStart();
			}
			PrintableSnapshot printableTag = new PrintableSnapshot(lastTag, level.intValue());
			enterSnapshot.setSecond(printableTag);
			printableTag.setEnd(snapshot);
			printable.add(printableTag);
		}
	}

	private void exit(Map<Integer, PrintablePair> levels, Snapshot snapshot, Integer level, PrintablePair enterSnapshot)
			throws IllegalStateException {
		if (enterSnapshot != null) {
			PrintableSnapshot printableSnapshot = enterSnapshot.getReference();
			printableSnapshot.setEnd(snapshot);
			Snapshot start = printableSnapshot.getStart();
			Group group = start.getGroup();
			if (group != null) {
				group.add(printableSnapshot);
			}
			levels.remove(level);
			String endTag = snapshot.getTag();
			String startTag = start.getTag();
			if (endTag != startTag && endTag != null && !endTag.equals(startTag)) {
				throw new IllegalStateException();
			}
		}
	}

	private void enter(Map<Integer, PrintablePair> levels, List<PrintableSnapshot> printable, Snapshot snapshot,
			Integer level, PrintablePair enterSnapshot) {
		if (enterSnapshot != null) {
			// We have received two enter in a row for the same level.
			enterSnapshot.getReference().setEnd(snapshot, Type.EXIT);
		}
		PrintablePair printableTag = new PrintablePair(snapshot, level);
		levels.put(level, printableTag);
		printable.add(printableTag.getReference());
	}
}