/*
 * Java
 *
 * Copyright 2014-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.basedriver.zwave;

import java.io.IOException;

import ej.basedriver.BinaryState;
import ej.basedriver.DryContact;
import ej.basedriver.PowerMeter;
import ej.basedriver.SwitchWithReturnState;
import ej.basedriver.impl.ControllerHelper;
import ej.basedriver.zwave.device.ZwaveDryContact;
import ej.basedriver.zwave.device.ZwaveMultilevelSensor;
import ej.basedriver.zwave.device.ZwavePowerMeter;
import ej.basedriver.zwave.device.ZwaveSwitch;
import ej.basedriver.zwave.device.ZwaveThermostat;
import ej.basedriver.zwave.device.ZwaveThermostatMode;
import ej.basedriver.zwave.frame.DataFrame;
import ej.basedriver.zwave.frame.SendData;
import ej.basedriver.zwave.util.NumberUtil;

/**
 * Handlers for the data.
 */
public class Handlers {
	/**
	 * Handle a controller frame.
	 *
	 * @param controller
	 *            the controller.
	 * @param dataFrame
	 *            the data frame.
	 * @throws IOException
	 *             Throws an io exception if a communication error occurred with the device.
	 */
	public static void handle(ZwaveController controller, DataFrame dataFrame) throws IOException {
		byte functionID = dataFrame.getFunctionId();
		switch (functionID) {
		case ZwaveProtocol.FUNC_ID_APPLICATION_COMMAND_HANDLER:
			applicationCommandHandler(controller, dataFrame);
			break;
		default:
			break;
		}
	}

	private static void applicationCommandHandler (ZwaveController controller, DataFrame dataFrame) throws IOException {
		byte [] bytes = dataFrame.getBytes();
		ZwaveNode node = controller.getNode(bytes[5]);
		if (node == null) {
			//error?
			return ;
		}

		byte commandClass = (byte) (bytes[7] & 0xFF) ;
		switch (commandClass) {
		case ZwaveProtocol.COMMAND_CLASS_WAKE_UP:
			applicationCommandHandlerWakeUp(controller, node, bytes);
			break;
		case ZwaveProtocol.COMMAND_CLASS_BASIC:
			applicationCommandHandlerBasic(controller, node, bytes);
			break;
		case ZwaveProtocol.COMMAND_CLASS_SWITCH_BINARY:
			applicationCommandHandlerSwitchBinary(controller, node, bytes);
			break;
		case ZwaveProtocol.COMMAND_CLASS_SENSOR_BINARY:
			applicationCommandHandlerSensorBinary(controller, node, bytes);
			break;
		case ZwaveProtocol.COMMAND_CLASS_SENSOR_MULTILEVEL:
			applicationCommandHandlerSensorMultilevel(controller, node, bytes);
			break;
		case ZwaveProtocol.COMMAND_CLASS_THERMOSTAT_MODE:
			applicationCommandHandlerThermostatMode(controller, node, bytes);
			break;
		case ZwaveProtocol.COMMAND_CLASS_THERMOSTAT_SETPOINT:
			applicationCommandHandlerThermostatSetPoint(controller, node, bytes);
			break;
		case ZwaveProtocol.COMMAND_CLASS_METER:
			applicationCommandHandlerMeter(controller, node, bytes);
			break;
		case ZwaveProtocol.COMMAND_CLASS_ASSOCIATION:
			applicationCommandHandlerAssociation(controller, node, bytes);
			break;
		default:
			break;
		}
	}

	private static void applicationCommandHandlerAssociation (ZwaveController controller, ZwaveNode node, byte [] bytes) throws IOException {
		byte command = (byte) (bytes[8] & 0xFF);
		switch (command) {
		case ZwaveProtocol.ASSOCIATION_GROUPINGS_REPORT:
			int groups = (bytes[9] & 0xFF);
			for (int cpt = 1; cpt <= groups; cpt++) {
				SendData frame = new SendData(node, ZwaveProtocol.COMMAND_CLASS_ASSOCIATION, ZwaveProtocol.ASSOCIATION_GET, 1);
				frame.addParameter((byte) cpt);
				controller.sendFrame(frame);
			}
			break;
		case ZwaveProtocol.ASSOCIATION_REPORT:
			int group = (bytes[9] & 0xFF);
			boolean found = false;
			for (int cpt = 11; cpt < bytes.length - 1; cpt++) {
				if (bytes[cpt] == controller.getId()) {
					found = true;
					break;
				}
			}
			if (!found) {
				SendData frame = new SendData(node, ZwaveProtocol.COMMAND_CLASS_ASSOCIATION, ZwaveProtocol.ASSOCIATION_SET, 2);
				frame.addParameter((byte) group);
				frame.addParameter(controller.getId());
				controller.sendFrame(frame);
			}
			break;
		default:
			break;
		}
	}

	private static void applicationCommandHandlerWakeUp (ZwaveController controller, ZwaveNode node, byte [] bytes) /*throws IOException*/ {
		byte command = (byte) (bytes[8] & 0xFF);
		switch (command) {
		case ZwaveProtocol.WAKE_UP_NOTIFICATION:
			if (ZwaveController.DEBUG) {
				System.out.println("Node " + node.getNodeUid() + " is awakened");
			}
			controller.setAwakeNode(node);
			break;
		case ZwaveProtocol.WAKE_UP_INTERVAL_REPORT:
			break;
		default:
			break;
		}
	}

	private static void changeSwitchState (ZwaveController controller, ZwaveSwitch zs, byte [] bytes) {
		if (bytes[9] == ZwaveSwitch.SWITCH_ON) {
			zs.setState(BinaryState.ON);
		} else {
			zs.setState(BinaryState.OFF);
		}

		zs.notifyComplete();
	}

	private static void applicationCommandHandlerSwitchBinary (ZwaveController controller, ZwaveNode node, byte [] bytes) /*throws IOException*/ {
		byte command = (byte) (bytes[8] & 0xFF) ;
		switch (command) {
		case ZwaveProtocol.SWITCH_BINARY_REPORT:
			ZwaveSwitch zs = (ZwaveSwitch) ControllerHelper.GetFirstChild(node, SwitchWithReturnState.class);
			if (zs == null) {
				//error?
				return ;
			}
			changeSwitchState(controller, zs, bytes);
			break;
		}

	}

	private static void applicationCommandHandlerSensorMultilevel(ZwaveController controller, ZwaveNode node,
			byte[] bytes) /* throws IOException */ {
		byte command = (byte) (bytes[8] & 0xFF);
		switch (command) {
		case ZwaveProtocol.SENSOR_MULTILEVEL_REPORT:
			ZwaveMultilevelSensor zms = ControllerHelper.GetFirstChild(node, ZwaveMultilevelSensor.class);
			if (zms == null) {
				// error?
				return;
			}
			int precision = ((bytes[10] & 0b11100000) >> 5);
			int size = (bytes[10] & 0b00000111);

			zms.setValue(NumberUtil.readDouble(bytes, 11, size, precision));
			break;
		}

	}

	private static void applicationCommandHandlerThermostatSetPoint(ZwaveController controller, ZwaveNode node,
			byte[] bytes) /* throws IOException */ {
		byte command = (byte) (bytes[8] & 0xFF);
		switch (command) {
		case ZwaveProtocol.THERMOSTAT_SETPOINT_REPORT:
			ZwaveThermostat zt = ControllerHelper.GetFirstChild(node, ZwaveThermostat.class);
			if (zt == null) {
				// error?
				return;
			}
			ZwaveThermostatMode mode = zt.getMode(Integer.valueOf(bytes[9] & 0x0F));
			if (mode != null) {
				int precision = ((bytes[10] & 0b11100000) >> 5);
				int size = (bytes[10] & 0b00000111);

				mode.updateValue(NumberUtil.readDouble(bytes, 11, size, precision));
			}
			break;
		}
	}

	private static void applicationCommandHandlerThermostatMode(ZwaveController controller, ZwaveNode node,
			byte[] bytes) /* throws IOException */ {
		byte command = (byte) (bytes[8] & 0xFF);
		ZwaveThermostat zt;
		switch (command) {
		case ZwaveProtocol.THERMOSTAT_MODE_REPORT:
			zt = ControllerHelper.GetFirstChild(node, ZwaveThermostat.class);
			if (zt == null) {
				// error?
				return;
			}
			zt.setMode(bytes, 7);
			break;
		case ZwaveProtocol.THERMOSTAT_MODE_SUPPORTED_REPORT:
			zt = ControllerHelper.GetFirstChild(node, ZwaveThermostat.class);
			if (zt == null) {
				// error?
				return;
			}
			zt.setModes(bytes, 9);
			break;
		}
	}

	private static void applicationCommandHandlerSensorBinary (ZwaveController controller, ZwaveNode node, byte [] bytes) /*throws IOException*/ {
		byte command = (byte) (bytes[8] & 0xFF) ;
		ZwaveDryContact zdc = (ZwaveDryContact) ControllerHelper.GetFirstChild(node, DryContact.class);
		if (zdc == null) {
			//error?
			return ;
		} else {
			switch (command) {
			case ZwaveProtocol.SENSOR_BINARY_REPORT:
				break;
			}
		}
	}

	private static void applicationCommandHandlerBasic (ZwaveController controller, ZwaveNode node, byte [] bytes) /*throws IOException*/ {
		byte command = (byte) (bytes[8] & 0xFF) ;
		switch (command) {
		case ZwaveProtocol.BASIC_SET:
			ZwaveDryContact zdc = (ZwaveDryContact) ControllerHelper.GetFirstChild(node, DryContact.class);
			if (zdc != null) {
				if (bytes[9] == 0x00) {
					zdc.setState(BinaryState.OFF);
				} else {
					zdc.setState(BinaryState.ON);
				}
			}
			break;
		default:
			break;
		}
	}

	private static void applicationCommandHandlerMeter (ZwaveController controller, ZwaveNode node, byte [] bytes) /*throws IOException*/ {
		byte command = (byte) (bytes[8] & 0xFF) ;
		ZwavePowerMeter zp = (ZwavePowerMeter) ControllerHelper.GetFirstChild(node, PowerMeter.class);
		if (zp == null) {
			//error?
			return;
		} else {
			switch (command) {
			case ZwaveProtocol.METER_REPORT:
				break;
			}
		}
	}
}
