/*
 * Java
 *
 * Copyright 2014-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.basedriver.zwave;

import java.io.IOException;
import java.io.OutputStream;

import ej.basedriver.zwave.frame.DataFrame;
import ej.basedriver.zwave.frame.RequestFrame;
import ej.basedriver.zwave.frame.RequestFrameListener.RequestFrameError;
import ej.basedriver.zwave.util.ZwaveLogger;

public class ZwaveSession {
	private static final long WAIT_ACK_TIMEOUT = 1600L;

	private final Ack ack;
	private RequestFrame lastFrame;
	private final ZwaveController controller;
	private final ZwaveLogger logger;

	public ZwaveSession(ZwaveController controller) {
		this.controller = controller;
		this.logger = controller.getLogger();
		ack = new Ack(logger);
	}

	/**
	 * @return
	 */
	public ZwaveController getController() {
		return controller;
	}

	public void receiveAck () {
		ack.receiveAck(true);
		logger.logReceiveAck();
	}

	public void receiveNak() {
		ack.receiveAck(false);
		logger.logReceiveNak();
	}

	public void receiveCan () {
		ack.receiveAck(false);
		logger.logReceiveCan();
	}

	public void session (RequestFrame frame, OutputStream outputStream) throws IOException, InterruptedException {
		this.lastFrame = frame;
		this.lastFrame.completeContent();

		if (ZwaveController.DEBUG) {
			System.out.println("Open session");
		}

		boolean error = true;

		error = ackWindow(outputStream);

		if (error) {
			lastFrame.notifyError(RequestFrameError.SEND_ERROR);
		} else {
			error = lastFrame.getSession().open();
			if (error) {
				lastFrame.notifyError(RequestFrameError.COMPLETE_ERROR);
			} else {
				lastFrame.notifyComplete();
			}
		}

		synchronized (this) {
			lastFrame = null;
		}
	}

	private boolean ackWindow (OutputStream outputStream) throws IOException, InterruptedException {
		ack.sendFrame(lastFrame, outputStream);

		while (true) {
			if (ack.timeoutExpired || ! ack.receiveAck) {
				if (lastFrame.getNumberRetransmission() <= ZwaveProtocol.MAX_RETRANSMISSION) {
					long waitBeforeRetransmission = 100 + lastFrame.getNumberRetransmission() * 1000;
					if (ack.timeoutExpired) {
						/* compensate the period we spent to wait for an ack */
						waitBeforeRetransmission = waitBeforeRetransmission - ZwaveProtocol.ACK_WAIT_PERIOD < 0 ? 0 : waitBeforeRetransmission - ZwaveProtocol.ACK_WAIT_PERIOD;
					}

					Thread.sleep(waitBeforeRetransmission);
					ack.sendFrame(lastFrame, outputStream);
					continue;
				} else {
					return true;
				}
			} else {
				return false;
			}
		}
	}

	public void receiveValidFrame(DataFrame frame) throws IOException {
		RequestFrame lastFrame = null;
		synchronized (this) {
			lastFrame = this.lastFrame;
		}

		if (lastFrame != null) {
			if (lastFrame.getFunctionId() == frame.getFunctionId()) {
				if (frame.getType() == ZwaveProtocol.REQUEST) {
					lastFrame.handleRequest(controller, frame);
				} else if (frame.getType() == ZwaveProtocol.RESPONSE) {
					lastFrame.handleResponse(controller, frame);
				} else {
					//error
				}

				return ;
			}
		}

		Handlers.handle(controller, frame);
	}

	static class Ack {
		boolean receiveAck;
		boolean timeoutExpired;
		ZwaveLogger logger;

		/**
		 * @param logger
		 */
		public Ack(ZwaveLogger logger) {
			this.logger = logger;
		}

		public synchronized void sendFrame (RequestFrame frame, OutputStream outputStream) throws IOException, InterruptedException {
			timeoutExpired = true;
			receiveAck = false;
			frame.send(outputStream);
			logger.logSendFrame(frame.getNumberRetransmission(), frame.getBytes());
			wait(WAIT_ACK_TIMEOUT);
		}

		public synchronized void receiveAck (boolean receiveAck) {
			this.receiveAck = receiveAck;
			timeoutExpired = false;
			notify();
		}
	}
}
