/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.components.dependencyinjection.util;

import ej.components.dependencyinjection.ServiceLoadingException;

/**
 * Helper class for service loaders.
 */
public class ServiceLoaderHelper {

	private ServiceLoaderHelper() {
		// utility class
	}

	/**
	 * Gets an instance of a class, based on its name.
	 *
	 * @param parent
	 *            the parent class.
	 * @param className
	 *            the name of the class to instantiate.
	 * @param <T>
	 *            the class type.
	 * @return an instance of the given class.
	 * @throws ServiceLoadingException
	 *             if an error occurs while instantiating the service implementation.
	 *
	 */
	public static <T> T createClassInstance(Class<T> parent, String className) throws ServiceLoadingException {
		try {
			Class<?> clazz = Class.forName(className);
			// ensure that is is the right type
			if (!parent.isAssignableFrom(clazz)) {
				throw new ClassCastException();
			}
			@SuppressWarnings("unchecked")
			T implementation = (T) clazz.newInstance(); // cast ensured by previous test

			return implementation;
		} catch (InstantiationException | IllegalAccessException | ClassNotFoundException e) {
			// can't find or instantiate implementation
			throw new ServiceLoadingException(parent.getName(), e);
		}
	}

}
