/*
 * Copyright 2013-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.components.registry.impl;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import ej.annotation.Nullable;
import ej.components.registry.BundleActivator;
import ej.components.registry.util.BundleRegistryHelper;

/**
 * Registry that loads the bundles from a file.
 * <p>
 * It needs only one parameter during initialization: the bundles file path. This file must contain a
 * {@link BundleActivator} list (classname and parameter pairs).
 * <p>
 * The classname is the fully qualified name of the {@link BundleActivator} implementation. The parameter is a string
 * that will be passed to the {@link #initialize()}, {@link #link()}, {@link #start()} and {@link #stop()} methods of
 * the {@link BundleActivator}. The classname and the parameter must be separated by a '<code>:</code>'.
 */
public class FileRegistry extends AbstractRegistry {

	private static final String FILE_EXTENSION = ".file"; //$NON-NLS-1$

	@Nullable
	private final String file;

	/**
	 * Creates a file registry.
	 */
	public FileRegistry() {
		String filename = FileRegistry.class.getName() + FILE_EXTENSION;
		this.file = System.getProperty(filename);
		if (this.file == null) {
			// missing bundles file path
			throw new IllegalArgumentException("Missing property " + filename); //$NON-NLS-1$
		}
	}

	/**
	 * Creates a file registry specifying the path to the file containing the list of bundles.
	 *
	 * @param file
	 *            the path to the bundles list file.
	 */
	public FileRegistry(String file) {
		this.file = file;
	}

	@Override
	protected BundleActivator[] loadBundles() {
		// read bundles file
		String file = this.file;
		assert file != null; // would have thrown an IllegalArgumentException in constructor
		return readFile(file);
	}

	private BundleActivator[] readFile(String file) {
		try (InputStream is = FileRegistry.class.getResourceAsStream(file)) {
			List<BundleActivator> resultList = new ArrayList<>();
			if (is != null) {
				boolean endOfFile = false;
				do {
					// read line
					StringBuilder lineBuilder = new StringBuilder();
					boolean dropComment = false;
					while (true) {
						int readByte = is.read();
						if (readByte == -1) {
							// end of file
							endOfFile = true;
							break;
						} else if (readByte == '\n') {
							// end of line
							break;
						} else {
							if (!dropComment) {
								if (readByte == '#') {
									dropComment = true;
								} else {
									lineBuilder.append((char) readByte);
								}
							}
						}
					}
					String line = lineBuilder.toString().trim();
					if (!line.isEmpty()) {
						BundleActivator bundle = BundleRegistryHelper.loadBundle(line);
						resultList.add(bundle);
					}
				} while (!endOfFile);
			}
			return resultList.toArray(new BundleActivator[resultList.size()]);
		} catch (IOException e) {
			throw new RuntimeException("Cannot read " + file, e); //$NON-NLS-1$
		}
	}

}
