/*
 * Java
 *
 * Copyright 2014-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.eventswatcher;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import ej.microui.event.EventGenerator;
import ej.microui.util.EventHandler;

/**
 * An events watcher aggregator groups several {@link EventsWatcher}s.
 */
public class EventsWatcherAggregator {

	private final Map<EventGenerator, EventsWatcher> interceptors;

	/**
	 * Creates an events watcher aggregator.
	 */
	public EventsWatcherAggregator() {
		this.interceptors = new HashMap<>();
	}

	/**
	 * Watches the events generated by an event generator.
	 * <p>
	 * All events generated by the given generator will be sent to the event handler.
	 *
	 * @param eventGenerator
	 *            the event generator to watch.
	 * @param eventHandler
	 *            the event handler to send the event to.
	 * @throws NullPointerException
	 *             if one of the given event generator or event handler is <code>null</code>.
	 */
	public void watch(EventGenerator eventGenerator, EventHandler eventHandler) {
		addAll(toList(eventGenerator), eventHandler, true);
	}

	/**
	 * Watches the events generated by a set of event generators.
	 *
	 * @param eventGenerators
	 *            the event generators to watch.
	 * @param eventHandler
	 *            the event handler to send the event to.
	 * @see #watch(EventGenerator, EventHandler)
	 * @throws NullPointerException
	 *             if one of the given event generators or event handler is <code>null</code>.
	 */
	public void watchAll(List<? extends EventGenerator> eventGenerators, EventHandler eventHandler) {
		addAll(eventGenerators, eventHandler, true);
	}

	/**
	 * Captures the events generated by an event generator.
	 * <p>
	 * All events generated by the given generator will be sent to the event handler.
	 * <p>
	 * The captured events will not be propagated to the original listener of the event generator (or the previous
	 * capture event handler).
	 *
	 * @param eventGenerator
	 *            the event generator to capture.
	 * @param eventHandler
	 *            the event handler to send the event to.
	 * @throws NullPointerException
	 *             if one of the given event generator or event handler is <code>null</code>.
	 */
	public void capture(EventGenerator eventGenerator, EventHandler eventHandler) {
		addAll(toList(eventGenerator), eventHandler, false);
	}

	/**
	 * Captures the events generated by an event generator.
	 *
	 * @param eventGenerators
	 *            the event generators to capture.
	 * @param eventHandler
	 *            the event handler to send the event to.
	 * @see #capture(EventGenerator, EventHandler)
	 * @throws NullPointerException
	 *             if one of the given event generators or event handler is <code>null</code>.
	 */
	public void captureAll(List<? extends EventGenerator> eventGenerators, EventHandler eventHandler) {
		addAll(eventGenerators, eventHandler, false);
	}

	/**
	 * Stops capturing the events for a specific event handler.
	 *
	 * @param eventHandler
	 *            the event handler to remove.
	 */
	public void uncapture(EventHandler eventHandler) {
		for (EventGenerator eventGenerator : this.interceptors.keySet()) {
			EventsWatcher interceptorsManager = this.interceptors.get(eventGenerator);
			if (interceptorsManager != null) {
				interceptorsManager.remove(eventHandler, false);
			}
		}
	}

	/**
	 * Stops watching the events for a specific event handler.
	 *
	 * @param eventHandler
	 *            the event handler to remove.
	 */
	public void unwatch(EventHandler eventHandler) {
		for (EventGenerator eventGenerator : this.interceptors.keySet()) {
			EventsWatcher interceptorsManager = this.interceptors.get(eventGenerator);
			if (interceptorsManager != null) {
				interceptorsManager.remove(eventHandler, true);
			}
		}
	}

	private void addAll(List<? extends EventGenerator> eventGenerators, EventHandler eventHandler, boolean watch) {
		if (eventHandler == null) {
			throw new NullPointerException();
		}

		for (EventGenerator eventGenerator : eventGenerators) {
			if (eventGenerator == null) {
				throw new NullPointerException();
			}
			EventsWatcher interceptorsManager;
			try {
				interceptorsManager = getWatchersManager(eventGenerator);
				interceptorsManager.add(eventHandler, watch);
			} catch (SecurityException e) {
				Logger.getAnonymousLogger().log(Level.WARNING, eventGenerator.getClass().getName() + " not watched.");
				// Ignore this event generator.
				continue;
			}
		}
	}

	private EventsWatcher getWatchersManager(EventGenerator eventGenerator) {
		EventsWatcher interceptorsManager = this.interceptors.get(eventGenerator);
		if (interceptorsManager == null) {
			interceptorsManager = new EventsWatcher(eventGenerator);
			this.interceptors.put(eventGenerator, interceptorsManager);
		}
		return interceptorsManager;
	}

	private static List<EventGenerator> toList(EventGenerator eventGenerator) {
		List<EventGenerator> list = new ArrayList<>();
		list.add(eventGenerator);
		return list;
	}
}
