/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.selector.combinator;

import ej.annotation.NonNull;
import ej.style.Element;
import ej.style.Selector;

/**
 * A general sibling combinator selects by checking the element sibling.
 * <p>
 * Equivalent to <code>element~element</code> selector in CSS.
 *
 * @see Element#getParentElement()
 */
public class GeneralSiblingCombinator extends Combinator {

	/**
	 * Creates a general sibling combinator.
	 *
	 * @param selector1
	 *            the first child.
	 * @param selector2
	 *            the second child.
	 */
	public GeneralSiblingCombinator(Selector selector1, Selector selector2) {
		super(selector1, selector2);
	}

	/**
	 * Creates a general sibling combinator.
	 *
	 * @param selectors
	 *            the list of selectors.
	 * @throws IllegalArgumentException
	 *             if there is less than two selectors.
	 */
	public GeneralSiblingCombinator(Selector... selectors) {
		super(selectors);
	}

	@Override
	public boolean fit(Element element) {
		Selector[] selectors = this.selectors;
		int size = selectors.length;
		int selectorIndex = size - 1;
		Selector ancestorSelector = selectors[selectorIndex--];
		if (!ancestorSelector.fit(element)) {
			return false;
		}

		Element parentElement = element.getParentElement();

		if (parentElement != null) {
			// Find element position in parent.
			Element[] elements = parentElement.getChildrenElements();
			int widgetsLength = elements.length;
			for (int i = -1; ++i < widgetsLength;) {
				Element elementCandidate = elements[i];

				if (elementCandidate == element) {
					// Index of widget is i.
					if (i <= selectorIndex) {
						return false;
					}

					ancestorSelector = selectors[selectorIndex--];
					while (i > 0) {
						@SuppressWarnings("null")
						@NonNull
						Element siblingElement = elements[--i];
						if (ancestorSelector.fit(siblingElement)) {
							if (selectorIndex == -1) {
								// That was the last one!
								return true;
							} else {
								// Next selector.
								ancestorSelector = selectors[selectorIndex--];
							}
						}
					}
					return false;
				}
			}
			// Should not be here.
			throw new AssertionError();
		}
		return false;
	}
}
