/*
 * Copyright 2018-2019 IS2T. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break IS2T warranties on the whole library.
 */
package ej.wadapps.application;

import java.security.Permission;

import ej.annotation.Nullable;

/**
 * Permission to manipulate applications using {@link Application#start()} or {@link Application#stop()} or
 * {@link ApplicationInstaller#install(java.io.InputStream)} or {@link ApplicationInstaller#uninstall(Application)}.
 */
public class ApplicationPermission extends Permission {

	private static final long serialVersionUID = 3364884452247889445L;

	/**
	 * Action associated with {@link Application#start()}.
	 */
	public static final String START_ACTION = "start"; //$NON-NLS-1$
	/**
	 * Action associated with {@link Application#stop()}.
	 */
	public static final String STOP_ACTION = "stop"; //$NON-NLS-1$
	/**
	 * Action associated with {@link ApplicationInstaller#install(java.io.InputStream)}.
	 */
	public static final String INSTALL_ACTION = "install"; //$NON-NLS-1$
	/**
	 * Action associated with {@link ApplicationInstaller#uninstall(Application)}.
	 */
	public static final String UNINSTALL_ACTION = "uninstall"; //$NON-NLS-1$

	private static final String EMPTY_STRING = ""; //$NON-NLS-1$

	private final String action;

	/**
	 * Creates a permission for the given application.
	 * <p>
	 * The action can be one of {@link #START_ACTION}, {@link #STOP_ACTION}, {@link #INSTALL_ACTION} or
	 * {@link #UNINSTALL_ACTION}.
	 * <p>
	 * The application may be {@code null} for the install action (the application is not created yet).
	 *
	 * @param application
	 *            the application.
	 * @param action
	 *            the action realized with the service.
	 */
	public ApplicationPermission(@Nullable Application application, String action) {
		super(computeName(application));
		this.action = action;
	}

	private static String computeName(@Nullable Application application) {
		if (application == null) {
			return EMPTY_STRING;
		} else {
			return application.getIdentifier() + '-' + application.getVersion();
		}
	}

	@Override
	public String getActions() {
		return this.action;
	}

	@Override
	public boolean equals(@Nullable Object obj) {
		return this == obj;
	}

	@Override
	public int hashCode() {
		return System.identityHashCode(this);
	}

	@Override
	public boolean implies(@Nullable Permission permission) {
		return equals(permission);
	}

}
