/*
 * Java
 *
 * Copyright 2015-2019 IS2T. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break IS2T warranties on the whole library.
 */
package ej.wadapps.multisandbox.application;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;

import com.microej.kf.util.KernelSupport;

import ej.annotation.Nullable;
import ej.kf.Feature;
import ej.kf.Kernel;
import ej.wadapps.application.Application;
import ej.wadapps.application.ApplicationOperationException;
import ej.wadapps.application.ApplicationPermission;

/**
 * Represents an application matching a KF feature.
 *
 * @see Feature
 */
public class FeatureApplication implements Application {

	private final Feature feature;
	private final String featureId;
	private final String featureVersion;

	/**
	 * Creates an application for a feature.
	 *
	 * @param feature
	 *            the feature to wrap.
	 */
	@SuppressWarnings("null")
	protected FeatureApplication(Feature feature) {
		this.feature = feature;
		this.featureId = feature.getName();
		this.featureVersion = feature.getVersion();
	}

	/**
	 * Gets the feature.
	 *
	 * @return the feature.
	 */
	public Feature getFeature() {
		return this.feature;
	}

	@Override
	public String getIdentifier() {
		return this.featureId;
	}

	@Override
	public String getVersion() {
		return this.featureVersion;
	}

	@Override
	public boolean equals(@Nullable Object obj) {
		Kernel.enter();
		if (this == obj) {
			return true;
		}

		if (!(obj instanceof Application)) {
			return false;
		}

		Application application = (Application) obj;
		return getIdentifier().equals(application.getIdentifier());
	}

	@Override
	public int hashCode() {
		Kernel.enter();
		return getIdentifier().hashCode() + getVersion().hashCode();
	}

	@Override
	public void start() throws IllegalStateException, SecurityException, ApplicationOperationException {
		start(new NullProgressMonitor());
	}

	@Override
	public void start(IProgressMonitor monitor) throws IllegalStateException, ApplicationOperationException {
		checkPermission(this, ApplicationPermission.START_ACTION);
		Kernel.enter();
		this.feature.start();
		// Kernel.exit();
	}

	@Override
	public void stop() throws IllegalStateException, ApplicationOperationException {
		stop(new NullProgressMonitor());
	}

	@Override
	public void stop(IProgressMonitor monitor) throws IllegalStateException, ApplicationOperationException {
		checkPermission(this, ApplicationPermission.STOP_ACTION);
		Kernel.enter();
		KernelSupport.stopFeature(this.feature);
		// Kernel.exit();
	}

	@Override
	public boolean isStarted() {
		Kernel.enter();
		switch (this.feature.getState()) {
		case STARTED:
			return true;
		case UNINSTALLED:
		case STOPPED:
		case INSTALLED:
			return false;
		default:
			throw new RuntimeException();
		}
		// Kernel.exit();
	}

	/**
	 * Checks that an action on a application is permitted.
	 *
	 * @param application
	 *            the application.
	 * @param action
	 *            the action.
	 */
	private void checkPermission(Application application, String action) {
		SecurityManager securityManager = System.getSecurityManager();
		if (securityManager != null) {
			securityManager.checkPermission(new ApplicationPermission(application, action));
		}
	}

}
