/*
 * Java
 *
 * Copyright 2014-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.watchdog;

import java.util.List;

import ej.components.dependencyinjection.ServiceLoaderFactory;
import ej.eventswatcher.EventsWatcherAggregator;
import ej.microui.event.EventGenerator;
import ej.microui.util.EventHandler;

/**
 * {@link EventsFeeder} feeds a watchdog when events are generated by all registered event generators.
 *
 * @see EventGenerator
 * @see Watchdog
 */
public class EventsFeeder implements EventHandler {

	/**
	 * The watchdog that will be notified on event.
	 */
	private final Watchdog watchdog;

	/**
	 * Generates activities from all the given generators.
	 *
	 * @param watchdog
	 *            the watchdog to feed.
	 * @see EventsWatcherAggregator#watchAll(List, EventHandler)
	 */
	public EventsFeeder(final Watchdog watchdog) {
		this.watchdog = watchdog;
	}

	/**
	 * Starts watching a list of generator.
	 *
	 * @param generators
	 *            the generator to watch.
	 */
	public void start(final List<EventGenerator> generators) {
		final EventsWatcherAggregator eventsWatcherAggregator = ServiceLoaderFactory.getServiceLoader()
				.getService(EventsWatcherAggregator.class, EventsWatcherAggregator.class);
		eventsWatcherAggregator.watchAll(generators, this);
	}

	/**
	 * Starts watching for all EventGenerator.
	 */
	public void start() {
		start(EventGenerator.getList(EventGenerator.class));
	}

	/**
	 * Stops watching the events.
	 */
	public void stop() {
		final EventsWatcherAggregator eventsWatcherAggregator = ServiceLoaderFactory.getServiceLoader()
				.getService(EventsWatcherAggregator.class, EventsWatcherAggregator.class);
		eventsWatcherAggregator.unwatch(this);
	}

	@Override
	public boolean handleEvent(final int event) {
		this.watchdog.feed();
		return false;
	}
}
