/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget;

import ej.annotation.Nullable;
import ej.basictool.ArrayTools;
import ej.giml.annotation.ElementChild;
import ej.microui.display.GraphicsContext;
import ej.mwt.Desktop;
import ej.mwt.MWT;
import ej.mwt.Panel;
import ej.mwt.Widget;
import ej.style.Element;
import ej.style.State;
import ej.style.Style;
import ej.style.background.Background;
import ej.style.container.AlignmentHelper;
import ej.style.container.Rectangle;
import ej.style.selector.InstanceSelector;
import ej.style.util.StyleHelper;

/**
 * A panel using a style specified in a style sheet.
 */
public class StyledPanel extends Panel implements Element, StyledRenderable {

	// Some (most) of the code in this class is duplicated in all StyledRenderable implementors for footprint
	// optimization (flash & RAM).

	/**
	 * The cached transparent state.
	 */
	private boolean transparent;
	/**
	 * The cached style.
	 */
	@Nullable
	private Style style;
	/**
	 * The class selectors.
	 */
	private String[] classSelectors;

	/**
	 * Creates a styled panel without class selector.
	 */
	public StyledPanel() {
		this.classSelectors = StyledHelper.EMPTY_STRING_ARRAY;
	}

	@Override
	public boolean isTransparent() {
		return this.transparent;
	}

	@Override
	public boolean contains(int x, int y) {
		return StyledHelper.contains(getX(), getY(), getWidth(), getHeight(), this.style, x, y);
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * If the given widget is not an element, an {@link IllegalArgumentException} is thrown.
	 *
	 * @throws IllegalArgumentException
	 *             if the given widget is not an element.
	 * @see Element
	 * @see StyledWidget
	 * @see StyledComposite
	 */
	@Override
	@ElementChild(prefix = "setWidget")
	public void setWidget(@Nullable Widget widget) throws IllegalArgumentException {
		if (widget == null || widget instanceof Element) {
			super.setWidget(widget);
		} else {
			throw new IllegalArgumentException();
		}
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * If the given desktop is not a styled desktop, an {@link IllegalArgumentException} is thrown.
	 *
	 * @throws IllegalArgumentException
	 *             if the given desktop is not an element.
	 * @see StyledDesktop
	 * @deprecated Use {@link #showAdjustingToChild(Desktop)} instead.
	 */
	@Override
	@Deprecated
	public void show(@Nullable Desktop desktop) throws NullPointerException {
		show(desktop, false);
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * If the given desktop is not a styled desktop, an {@link IllegalArgumentException} is thrown.
	 *
	 * @throws IllegalArgumentException
	 *             if the given desktop is not an element.
	 * @see StyledDesktop
	 * @deprecated Use {@link #showAdjustingToChild(Desktop)} or {@link #showFullScreen(Desktop)} instead.
	 */
	@Override
	@Deprecated
	public void show(@Nullable Desktop desktop, boolean fill) throws NullPointerException {
		if (desktop instanceof StyledDesktop) {
			super.show(desktop, fill);
		} else if (desktop == null) {
			throw new NullPointerException();
		} else {
			throw new IllegalArgumentException();
		}
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * If the given desktop is not a styled desktop, an {@link IllegalArgumentException} is thrown.
	 *
	 * @throws IllegalArgumentException
	 *             if the given desktop is not an element.
	 * @see StyledDesktop
	 */
	@Override
	public void showFullScreen(@Nullable Desktop desktop) throws NullPointerException {
		if (desktop instanceof StyledDesktop) {
			super.showFullScreen(desktop);
		} else if (desktop == null) {
			throw new NullPointerException();
		} else {
			throw new IllegalArgumentException();
		}
	}

	/**
	 * Requests the panel to be shown on the specified desktop.
	 * <p>
	 * The panel is fitted to its child widget size. Its location is set according to its style alignment.
	 * <p>
	 * If the desktop is shown, the panel is automatically validated.
	 * <p>
	 * The panel is added to the list of panels known by the desktop.
	 * <p>
	 * If the panel is already shown on another desktop, it is hidden on this desktop before being shown on the new
	 * <p>
	 * If the given desktop is not a styled desktop, an {@link IllegalArgumentException} is thrown.
	 *
	 * @param desktop
	 *            the desktop
	 * @throws NullPointerException
	 *             if the given desktop is <code>null</code>.
	 * @throws IllegalArgumentException
	 *             if the given desktop is not an element.
	 * @see StyledDesktop
	 * @see Style#getAlignment()
	 * @see #hide()
	 * @see #isShown()
	 * @see #revalidate()
	 * @since 2.2
	 */
	@Override
	public void showAdjustingToChild(@Nullable Desktop desktop) throws NullPointerException {
		if (desktop instanceof StyledDesktop) {
			super.showAdjustingToChild(desktop);
		} else if (desktop == null) {
			throw new NullPointerException();
		} else {
			throw new IllegalArgumentException();
		}
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * If the given desktop is not a styled desktop, an {@link IllegalArgumentException} is thrown.
	 *
	 * @throws IllegalArgumentException
	 *             if the given desktop is not an element.
	 * @see StyledDesktop
	 */
	@Override
	public void showUsingBounds(@Nullable Desktop desktop) throws NullPointerException {
		if (desktop instanceof StyledDesktop) {
			super.showUsingBounds(desktop);
		} else if (desktop == null) {
			throw new NullPointerException();
		} else {
			throw new IllegalArgumentException();
		}
	}

	@Override
	@Nullable
	public StyledDesktop getDesktop() {
		return (StyledDesktop) super.getDesktop();
	}

	@Override
	@Nullable
	public Element getParentElement() {
		return getDesktop();
	}

	@Override
	public Element[] getChildrenElements() {
		Widget widget = getWidget();
		if (widget != null) {
			return new Element[] { (Element) widget };
		}
		return EMPTY_ELEMENT_ARRAY;
	}

	@Override
	public Element getChild(int index) {
		if (index == 0) {
			Widget widget = getWidget();
			if (widget != null) {
				return (Element) widget;
			}
		}
		throw new IndexOutOfBoundsException();
	}

	@Override
	public int getChildrenCount() {
		if (getWidget() == null) {
			return 0;
		} else {
			return 1;
		}
	}

	@Override
	public void mergeStyle(Style style) {
		StyleHelper.getStylesheet().addRule(new InstanceSelector(this), style);
		updateStyle();
	}

	@Override
	public void removeInstanceStyle() {
		StyleHelper.getStylesheet().removeRule(new InstanceSelector(this));
		updateStyle();
	}

	@Override
	public Style getStyle() {
		Style style = this.style;
		if (style == null) {
			style = StyledHelper.getStyleFromStylesheet(this);
			setStyle(style);
		}
		return style;
	}

	private void setStyle(Style newStyle) {
		this.style = newStyle;
		Background background = newStyle.getBackground();
		this.transparent = background.isTransparent();
	}

	/**
	 * Updates the style of the widget.
	 */
	protected void updateStyle() {
		// Do not update style if not shown. See validate method.
		if (isShown() && updateStyleOnly()) {
			Widget child = getWidget();
			if (child instanceof StyledComposite) {
				((StyledComposite) child).updateStyle();
			} else if (child instanceof StyledWidget) {
				((StyledWidget) child).updateStyle();
			}
			repaint();
		}
	}

	/**
	 * Update widget style without repainting it.
	 *
	 * @return <code>true</code> if the style has been updated, <code>false</code> otherwise.
	 */
	protected boolean updateStyleOnly() {
		Style newStyle = StyledHelper.getStyleFromStylesheet(this);

		if (!newStyle.equals(this.style)) {
			setStyle(newStyle);
			return true;
		}
		return false;
	}

	@Override
	public void render(GraphicsContext g) {
		StyledHelper.render(g, this);
	}

	@Override
	public void renderContent(GraphicsContext g, Style style, Rectangle bounds) {
		// Do nothing by default.
	}

	@Override
	public void validate() {
		super.validate();
		StyledDesktop desktop = getDesktop();
		if (isAdjustedToChild() && desktop != null) {
			Style style = getStyle();
			int alignment = style.getAlignment();
			int x = AlignmentHelper.computeXLeftCorner(getWidth(), 0, desktop.getWidth(), alignment);
			int y = AlignmentHelper.computeYTopCorner(getHeight(), 0, desktop.getHeight(), alignment);
			setLocation(x, y);
		}
	}

	@Override
	public void validate(int widthHint, int heightHint) {
		updateStyleOnly();
		Rectangle bounds = StyledHelper.validate(widthHint, heightHint, this);
		setPreferredSize(bounds.getWidth(), bounds.getHeight());
	}

	@Override
	public Rectangle validateContent(Style style, Rectangle bounds) {
		Widget widget = getWidget();
		int widthHint = bounds.getWidth();
		int heightHint = bounds.getHeight();
		if (widget != null) {
			widget.validate(widthHint, heightHint);

			if (widthHint == MWT.NONE) {
				widthHint = widget.getPreferredWidth();
			}
			if (heightHint == MWT.NONE) {
				heightHint = widget.getPreferredHeight();
			}

			widget.setBounds(bounds.getX(), bounds.getY(), widthHint, heightHint);
		}
		return new Rectangle(0, 0, widthHint, heightHint);
	}

	@Override
	public boolean hasClassSelector(String classSelector) {
		return ArrayTools.containsEquals(this.classSelectors, classSelector);
	}

	@Override
	public void addClassSelector(String classSelector) {
		this.classSelectors = StyledHelper.addClassSelector(this.classSelectors, classSelector);
		updateStyle();
	}

	@Override
	public void removeClassSelector(String classSelector) {
		int classSelectorsLength = this.classSelectors.length;
		this.classSelectors = ArrayTools.remove(this.classSelectors, classSelector);
		if (this.classSelectors.length != classSelectorsLength) {
			updateStyle();
		}
	}

	@Override
	public void setClassSelectors(String classSelector) {
		this.classSelectors = StyledHelper.setClassSelectors(classSelector);
		updateStyle();
	}

	@Override
	public void removeAllClassSelectors() {
		if (this.classSelectors.length != 0) {
			this.classSelectors = StyledHelper.EMPTY_STRING_ARRAY;
			updateStyle();
		}
	}

	@Override
	public boolean isInState(State state) {
		return false;
	}

	@Override
	@Nullable
	public String getAttribute(String attribute) {
		return null;
	}

}
