/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.basic;

import ej.annotation.Nullable;
import ej.giml.annotation.Element;
import ej.giml.annotation.ElementConstructor;
import ej.giml.annotation.ElementRequiredAttribute;
import ej.style.State;
import ej.style.util.StyleHelper;
import ej.widget.listener.OnClickListener;
import ej.widget.util.ButtonHelper;
import ej.widget.util.GenericListener;

/**
 * A button image is a widget that displays an image and reacts to click events.
 *
 * @see Image
 * @since 2.3.0
 */
@Element
public class ButtonImage extends Image implements GenericListener {

	private final ButtonHelper buttonHelper;

	/**
	 * Creates a button image without source.
	 */
	public ButtonImage() {
		this((ej.microui.display.Image) null);
	}

	/**
	 * Creates a button image with the path to the source to display.
	 *
	 * @param source
	 *            the path to the source to display.
	 * @throws NullPointerException
	 *             if the given source is <code>null</code>.
	 */
	@ElementConstructor
	public ButtonImage(@ElementRequiredAttribute String source) {
		this(StyleHelper.getImage(source));
	}

	/**
	 * Creates a button image with the source to display.
	 *
	 * @param source
	 *            the source to display.
	 */
	public ButtonImage(@Nullable ej.microui.display.Image source) {
		super(source);
		setEnabled(true);
		this.buttonHelper = new ButtonHelper(this);
	}

	@Override
	public void update() {
		updateStyle();
	}

	/**
	 * Performs the actions associated to a click.
	 */
	public void performClick() {
		this.buttonHelper.performClick();
	}

	/**
	 * Adds a listener on the click events of the button.
	 *
	 * @param listener
	 *            the listener to add.
	 * @throws NullPointerException
	 *             if the given listener is <code>null</code>.
	 */
	public void addOnClickListener(OnClickListener listener) {
		this.buttonHelper.addOnClickListener(listener);
	}

	/**
	 * Removes a listener on the click events of the button.
	 *
	 * @param listener
	 *            the listener to remove.
	 */
	public void removeOnClickListener(OnClickListener listener) {
		this.buttonHelper.removeOnClickListener(listener);
	}

	@Override
	public boolean isInState(State state) {
		return (state == State.Active && this.buttonHelper.isPressed()) || super.isInState(state);
	}

	@Override
	public boolean handleEvent(int event) {
		if (this.buttonHelper.handleEvent(event)) {
			return true;
		}
		return super.handleEvent(event);
	}

}
