/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.basic.image;

import ej.microui.display.GraphicsContext;
import ej.microui.display.Image;
import ej.style.Style;
import ej.style.container.Rectangle;
import ej.style.util.StyleHelper;
import ej.widget.basic.AbstractProgress;
import ej.widget.model.BoundedRangeModel;
import ej.widget.model.DefaultBoundedRangeModel;

/**
 * A progress using some images to render.
 */
public class ImageProgress extends AbstractProgress {

	private boolean horizontal;

	/**
	 * Creates a progress with the given model.
	 *
	 * @param model
	 *            the model to use.
	 */
	public ImageProgress(BoundedRangeModel model) {
		super(model);
		this.horizontal = true;
	}

	/**
	 * Creates a progress with a default bounded range as model.
	 *
	 * @param min
	 *            the minimum value of the progress bar.
	 * @param max
	 *            the maximum value of the progress bar.
	 * @param initialValue
	 *            the initial value of the progress bar.
	 * @see DefaultBoundedRangeModel
	 */
	public ImageProgress(int min, int max, int initialValue) {
		super(min, max, initialValue);
		this.horizontal = true;
	}

	/**
	 * Sets the slider orientation: horizontal or vertical.
	 *
	 * @param horizontal
	 *            <code>true</code> to set the slider horizontal, <code>false</code> to set the slider vertical.
	 */
	public void setHorizontal(boolean horizontal) {
		this.horizontal = horizontal;
	}

	@Override
	public void renderContent(GraphicsContext g, Style style, Rectangle bounds) {
		int width = bounds.getWidth();
		int height = bounds.getHeight();
		int foregroundColor = style.getForegroundColor();
		g.setColor(foregroundColor);

		float percentComplete = getPercentComplete();

		if (this.horizontal) {
			Image bar = getBarImage(true);
			int barWidth = bar.getWidth();
			int xbar = (width - barWidth) >> 1;
			int regionWidth = (int) (percentComplete * barWidth);
			g.drawRegion(bar, 0, 0, regionWidth, bar.getHeight(), xbar, height >> 1,
					GraphicsContext.LEFT | GraphicsContext.VCENTER);
		} else {
			Image bar = getBarImage(false);
			int barHeight = bar.getHeight();
			int ybar = (height - barHeight) >> 1;
			int regionHeight = (int) (percentComplete * barHeight);
			g.drawRegion(bar, 0, 0, bar.getWidth(), regionHeight, width >> 1, ybar,
					GraphicsContext.HCENTER | GraphicsContext.TOP);
		}
	}

	private Image getBarImage(boolean horizontal) {
		String imagePath = horizontal ? ImageHelper.getTheme().getProgressBarHorizontalPath()
				: ImageHelper.getTheme().getProgressBarVerticalPath();
		Image image = StyleHelper.getImage(imagePath);
		if (image == null) {
			throw new NullPointerException();
		}
		return image;
	}

	@Override
	public Rectangle validateContent(Style style, Rectangle availableSize) {
		Image bar = getBarImage(this.horizontal);
		return new Rectangle(0, 0, bar.getWidth(), bar.getHeight());
	}

}
