/*
 * Java
 *
 * Copyright 2017-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.container;

import java.util.Iterator;

import ej.microui.display.Display;
import ej.mwt.Widget;
import ej.style.container.Rectangle;
import ej.widget.basic.PagingIndicator;
import ej.widget.util.SwipeEventHandler;

/**
 * A fill carousel is a composite that contains a list of widget. Each widget takes the size of the carousel, it must be
 * be scrolled to access the other widgets.
 *
 * @see AbstractCarousel
 * @since 2.3.0
 */
public class FillCarousel extends AbstractCarousel {

	private int totalSize;

	private int position;

	/**
	 * Creates a fill carousel without cursor.
	 *
	 * @param horizontal
	 *            <code>true</code> if the carousel is horizontal, <code>false</code> otherwise.
	 * @param cyclic
	 *            <code>true</code> if the carousel is cyclic, <code>false</code> otherwise.
	 */
	public FillCarousel(boolean horizontal, boolean cyclic) {
		super(horizontal, cyclic);
	}

	/**
	 * Creates a fill carousel with a cursor that can be hidden automatically or not.
	 *
	 * @param horizontal
	 *            <code>true</code> if the carousel is horizontal, <code>false</code> otherwise.
	 * @param cyclic
	 *            <code>true</code> if the carousel is cyclic, <code>false</code> otherwise.
	 * @param cursor
	 *            the cursor that indicate the progress in the carousel.
	 * @param autoHideCursor
	 *            <code>true</code> if the cursor hides automatically, <code>false</code> otherwise.
	 */
	public FillCarousel(boolean horizontal, boolean cyclic, PagingIndicator cursor, boolean autoHideCursor) {
		super(horizontal, cyclic, cursor, autoHideCursor);
	}

	@Override
	protected Rectangle validateChildren(int boundsWidth, int boundsHeight, boolean horizontal) {
		int width = boundsWidth;
		int height = boundsHeight;
		Iterator<Widget> iterator = iterator();
		// Each child takes the full height and the width it needs when horizontal, the opposite when vertical.
		while (iterator.hasNext()) {
			Widget widget = iterator.next();
			if (widget != this.cursor) {
				widget.validate(boundsWidth, boundsHeight);
				width = Math.max(width, widget.getPreferredWidth());
				height = Math.max(height, widget.getPreferredHeight());
			}
		}
		return new Rectangle(0, 0, width, height);
	}

	@Override
	protected SwipeEventHandler setBoundsChildren(Rectangle bounds, int widgetsCount) {
		int boundsX = bounds.getX();
		int boundsY = bounds.getY();
		int boundsWidth = bounds.getWidth();
		int boundsHeight = bounds.getHeight();

		PagingIndicator cursor = this.cursor;
		boolean horizontal = this.horizontal;

		int currentX = boundsX;
		int currentY = boundsY;
		Iterator<Widget> iterator = iterator();
		// Each child takes the full height and the width it needs when horizontal, the opposite when vertical.
		while (iterator.hasNext()) {
			Widget widget = iterator.next();
			if (widget != cursor) {
				widget.setBounds(currentX, currentY, boundsWidth, boundsHeight);
				if (horizontal) {
					currentX += boundsWidth;
				} else {
					currentY += boundsHeight;
				}
			}
		}

		// Compute swipe size.
		int boundsSize;
		if (horizontal) {
			boundsSize = boundsWidth;
			this.totalSize = currentX - boundsX;
		} else {
			boundsSize = boundsHeight;
			this.totalSize = currentY - boundsY;
		}

		this.position = 0;
		return new SwipeEventHandler(widgetsCount, boundsSize, this.cyclic, true, horizontal, this);
	}

	@Override
	public void onMove(final int position) {
		Display.getDefaultDisplay().callSerially(new Runnable() {
			@Override
			public void run() {
				onMoveInternal(position);
			}
		});
		repaint();
	}

	private void onMoveInternal(int position) {
		boolean horizontal = this.horizontal;

		int totalSize = this.totalSize;
		boolean cyclic = this.cyclic;
		int shift = position - this.position;
		PagingIndicator cursor = this.cursor;
		int widgetsCount = getWidgetsCount();
		if (cursor != null) {
			widgetsCount--;
		}

		// Search for the currently selected widget.
		int selectedIndex = position * widgetsCount / totalSize;
		float percent = 1f - ((float) ((position * widgetsCount) % totalSize)) / totalSize;
		int focusIndex;
		// Focus the nearest widget.
		if (!cyclic) {
			if (percent > 1f) {
				// Before the first.
				selectedIndex = 0;
				percent = 1f;
			} else if (selectedIndex >= widgetsCount - 1 && percent < 1f) {
				// After the last.
				selectedIndex = widgetsCount - 1;
				percent = 1;
			}
		}
		if (percent > 0.5f) {
			focusIndex = selectedIndex;
		} else {
			focusIndex = (selectedIndex + 1) % widgetsCount;
		}

		Iterator<Widget> iterator = iterator();
		// Move all children according to the move.
		int index = 0;
		while (iterator.hasNext()) {
			Widget widget = iterator.next();
			if (widget != cursor) {
				moveWidget(widget, horizontal, totalSize, cyclic, shift);
				if (index == focusIndex) {
					// A cursor flicker is caused by the request focus because the widgets than gain and lost the focus
					// are repainted. It can be solved by a management of the z-index (the cursor is always repaint
					// above the other ones).
					widget.requestFocus();
				}
				index++;
			}
		}
		setSelectedIndex(selectedIndex);
		if (cursor != null) {
			cursor.setSelectedItem(selectedIndex, percent);
		}

		this.position = position;
	}

}
