/*
 * Java
 *
 * Copyright 2009-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.container;

import java.util.Iterator;

import ej.giml.annotation.Element;
import ej.giml.annotation.ElementAttribute;
import ej.giml.annotation.ElementChild;
import ej.giml.annotation.ElementConstructor;
import ej.mwt.MWT;
import ej.mwt.Widget;
import ej.style.Style;
import ej.style.container.Rectangle;
import ej.widget.StyledComposite;

/**
 * Lays out its children in rows or columns. Fills the current line (row or column) until there is no more space, then
 * create a new line and repeat the process.
 * <p>
 * Horizontal:
 * <p>
 * <img src="doc-files/flow_horizontal.png" alt="Horizontal flow.">
 * <p>
 * Vertical:
 * <p>
 * <img src="doc-files/flow_vertical.png" alt="Vertical flow.">
 */
@Element
public class Flow extends StyledComposite {

	/**
	 * Lay out direction.
	 */
	protected boolean horizontal;

	/**
	 * Creates a horizontal flow composite.
	 */
	public Flow() {
		this.horizontal = true;
	}

	/**
	 * Creates a flow composite specifying its orientation.
	 *
	 * @param horizontal
	 *            <code>true</code> to set the composite horizontal, <code>false</code> to set the composite vertical.
	 */
	@ElementConstructor
	public Flow(@ElementAttribute(defaultValue = "true") boolean horizontal) {
		this.horizontal = horizontal;
	}

	/**
	 * Sets the composite orientation: horizontal or vertical.
	 *
	 * @param horizontal
	 *            <code>true</code> to set the composite horizontal, <code>false</code> to set the composite vertical.
	 */
	public void setHorizontal(boolean horizontal) {
		this.horizontal = horizontal;
		invalidate();
	}

	@Override
	@ElementChild(cardinality = Integer.MAX_VALUE, ordered = true)
	public void add(Widget widget) throws NullPointerException, IllegalArgumentException {
		super.add(widget);
	}

	@Override
	public void remove(Widget widget) throws NullPointerException {
		super.remove(widget);
	}

	@Override
	public void removeAllWidgets() {
		super.removeAllWidgets();
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Lays out children in rows if horizontal, or in columns if vertical.
	 * <p>
	 * If the width (resp. height) of the row (resp. column) is too small for all the components, it creates several
	 * lines (resp. columns). The size of each child is set to the child's preferred size.
	 */
	@Override
	public Rectangle validateContent(Style style, Rectangle bounds) {
		int widthHint = bounds.getWidth();
		int heightHint = bounds.getHeight();

		boolean computeWidth = widthHint == MWT.NONE;
		boolean computeHeight = heightHint == MWT.NONE;

		boolean horizontalLocal = this.horizontal;

		int currentX = 0;
		int currentY = 0;
		int currentLineHeight = 0;
		int currentColumnWidth = 0;

		// System.out.println("FlowComposite.validate()");
		Iterator<Widget> iterator = iterator();
		while (iterator.hasNext()) {
			Widget widget = iterator.next();

			// System.out.println("\t* widget: " + widget);
			widget.validate(MWT.NONE, MWT.NONE);

			int widgetPreferredWidth = widget.getPreferredWidth();
			int widgetPreferredHeight = widget.getPreferredHeight();
			// System.out.println("\tpreferred size: " + widgetPreferredWidth + " " + widgetPreferredHeight);

			if (horizontalLocal) {
				if (createNew(computeWidth, currentX, widgetPreferredWidth, widthHint)) {
					// System.out.println("\tnew line");
					// Create a new line.
					currentX = 0;
					currentY += currentLineHeight;
					currentLineHeight = 0;
				}
				currentX += widgetPreferredWidth;
				currentLineHeight = Math.max(currentLineHeight, widgetPreferredHeight);
			} else {
				if (createNew(computeHeight, currentY, widgetPreferredHeight, heightHint)) {
					// System.out.println("\tnew column");
					// Create a new column.
					currentY = 0;
					currentX += currentColumnWidth;
					currentColumnWidth = 0;
				}
				currentY += widgetPreferredHeight;
				currentColumnWidth = Math.max(currentColumnWidth, widgetPreferredWidth);
			}
		}

		if (computeWidth) {
			widthHint = currentX + currentColumnWidth;
		}
		if (computeHeight) {
			heightHint = currentY + currentLineHeight;
		}

		return new Rectangle(0, 0, widthHint, heightHint);
	}

	private boolean createNew(boolean computeSize, int currentPixel, int widgetPreferredSize, int sizeHint) {
		return !computeSize && currentPixel + widgetPreferredSize > sizeHint;
	}

	@Override
	protected void setBoundsContent(Rectangle bounds) {
		int x = bounds.getX();
		int y = bounds.getY();
		int width = bounds.getWidth();
		int height = bounds.getHeight();

		int currentX = 0;
		int currentY = 0;
		int currentLineHeight = 0;
		int currentColumnWidth = 0;

		boolean horizontalLocal = this.horizontal;

		// System.out.println("FlowComposite.setBounds()");
		Iterator<Widget> iterator = iterator();
		while (iterator.hasNext()) {
			Widget widget = iterator.next();

			int widgetPreferredWidth = widget.getPreferredWidth();
			int widgetPreferredHeight = widget.getPreferredHeight();

			// System.out.println("\t* widget: " + widget);
			// System.out.println(
			// "\tat " + currentX + " " + currentY + " " + widgetPreferredWidth + " " + widgetPreferredHeight);

			if (horizontalLocal) {
				if (currentX + widgetPreferredWidth > width) {
					// create a new line
					currentX = 0;
					currentY += currentLineHeight;
					currentLineHeight = 0;
				}
				widget.setBounds(x + currentX, y + currentY, widgetPreferredWidth, widgetPreferredHeight);
				currentX += widgetPreferredWidth;
				currentLineHeight = Math.max(currentLineHeight, widgetPreferredHeight);
			} else {
				if (currentY + widgetPreferredHeight > height) {
					// create a new column
					currentY = 0;
					currentX += currentColumnWidth;
					currentColumnWidth = 0;
				}
				widget.setBounds(x + currentX, y + currentY, widgetPreferredWidth, widgetPreferredHeight);
				currentY += widgetPreferredHeight;
				currentColumnWidth = Math.max(currentColumnWidth, widgetPreferredWidth);
			}
		}
	}

}
