/*
 * Java
 *
 * Copyright 2017-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.container;

import java.util.Iterator;

import ej.microui.display.Display;
import ej.mwt.MWT;
import ej.mwt.Widget;
import ej.style.container.Rectangle;
import ej.widget.basic.PagingIndicator;
import ej.widget.util.SwipeEventHandler;

/**
 * A list carousel is a composite that contains a list of widget. The widgets are ordered from top to bottom. If the
 * widgets exceed the size of the carousel, they are scrolled.
 *
 * @see AbstractCarousel
 * @since 2.3.0
 */
public class ListCarousel extends AbstractCarousel {

	private boolean needToScroll;
	private int swipeSize;
	private int totalSize;
	private int totalSizeMinusLast;

	private int position;

	/**
	 * Creates a list carousel without cursor.
	 *
	 * @param horizontal
	 *            <code>true</code> if the carousel is horizontal, <code>false</code> otherwise.
	 * @param cyclic
	 *            <code>true</code> if the carousel is cyclic, <code>false</code> otherwise.
	 */
	public ListCarousel(boolean horizontal, boolean cyclic) {
		super(horizontal, cyclic);
	}

	/**
	 * Creates a list carousel with a cursor that can be hidden automatically or not.
	 *
	 * @param horizontal
	 *            <code>true</code> if the carousel is horizontal, <code>false</code> otherwise.
	 * @param cyclic
	 *            <code>true</code> if the carousel is cyclic, <code>false</code> otherwise.
	 * @param cursor
	 *            the cursor that indicate the progress in the carousel.
	 * @param autoHideCursor
	 *            <code>true</code> if the cursor hides automatically, <code>false</code> otherwise.
	 */
	public ListCarousel(boolean horizontal, boolean cyclic, PagingIndicator cursor, boolean autoHideCursor) {
		super(horizontal, cyclic, cursor, autoHideCursor);
	}

	@Override
	protected Rectangle validateChildren(int boundsWidth, int boundsHeight, boolean horizontal) {
		int width;
		int height;
		if (horizontal) {
			width = 0;
			height = boundsHeight;
		} else {
			width = boundsWidth;
			height = 0;
		}
		Iterator<Widget> iterator = iterator();
		// Each child takes the full height and the width it needs when horizontal, the opposite when vertical.
		while (iterator.hasNext()) {
			Widget widget = iterator.next();
			if (widget != this.cursor) {
				if (horizontal) {
					widget.validate(MWT.NONE, boundsHeight);
					width += widget.getPreferredWidth();
					height = Math.max(height, widget.getPreferredHeight());
				} else {
					widget.validate(boundsWidth, MWT.NONE);
					width = Math.max(width, widget.getPreferredWidth());
					height += widget.getPreferredHeight();
				}
			}
		}
		return new Rectangle(0, 0, width, height);
	}

	@Override
	protected SwipeEventHandler setBoundsChildren(Rectangle bounds, int widgetsCount) {
		int boundsX = bounds.getX();
		int boundsY = bounds.getY();
		int boundsWidth = bounds.getWidth();
		int boundsHeight = bounds.getHeight();

		PagingIndicator cursor = this.cursor;
		boolean horizontal = this.horizontal;

		int currentX = boundsX;
		int currentY = boundsY;
		int[] sizes = new int[widgetsCount];
		int index = 0;
		Iterator<Widget> iterator = iterator();
		// Each child takes the full height and the width it needs when horizontal, the opposite when vertical.
		while (iterator.hasNext()) {
			Widget widget = iterator.next();
			if (widget != cursor) {
				int widgetPreferredWidth = widget.getPreferredWidth();
				int widgetPreferredHeight = widget.getPreferredHeight();
				if (horizontal) {
					widget.setBounds(currentX, currentY, widgetPreferredWidth, boundsHeight);
					sizes[index++] = widgetPreferredWidth;
					currentX += widgetPreferredWidth;
				} else {
					widget.setBounds(currentX, currentY, boundsWidth, widgetPreferredHeight);
					sizes[index++] = widgetPreferredHeight;
					currentY += widgetPreferredHeight;
				}
			}
		}
		// Compute swipe size.
		int excess;
		int size;
		if (horizontal) {
			size = currentX - boundsX;
			excess = size - boundsWidth;
		} else {
			size = currentY - boundsY;
			excess = size - boundsHeight;
		}

		this.needToScroll = excess > 0;
		this.swipeSize = excess;
		this.totalSize = size;
		this.totalSizeMinusLast = size - sizes[widgetsCount - 1];
		this.position = 0;

		return new SwipeEventHandler(sizes, this.cyclic, true, horizontal, this);
	}

	@Override
	public synchronized void onMove(final int position) {
		Display.getDefaultDisplay().callSerially(new Runnable() {
			@Override
			public void run() {
				onMoveInternal(position);
			}
		});
		repaint();
	}

	private void onMoveInternal(int position) {
		boolean horizontal = this.horizontal;

		int currentSize = 0;
		int totalSize = this.totalSize;
		boolean cyclic = this.cyclic;
		int relativePosition;
		if (cyclic) {
			// If cyclic, the selected item is always on top.
			relativePosition = position;
		} else {
			// If not cyclic, the selected item is on top at the beginning but on bottom at the end.
			relativePosition = position * this.swipeSize / this.totalSizeMinusLast;
		}
		int shift;
		if (this.needToScroll) {
			shift = relativePosition - this.position;
		} else {
			shift = 0;
		}
		PagingIndicator cursor = this.cursor;
		int widgetsCount = getWidgetsCount();
		if (cursor != null) {
			widgetsCount--;
		}
		Iterator<Widget> iterator = iterator();
		// Move all children according to the move.
		int selectedIndex = Integer.MIN_VALUE;
		float percent = 1f;
		int index = 0;
		int previousSize = 0;
		while (iterator.hasNext()) {
			Widget widget = iterator.next();
			if (widget != cursor) {
				int widgetSize = moveWidget(widget, horizontal, totalSize, cyclic, shift);

				// Search for the currently selected item.
				int halfWidget = currentSize + widgetSize / 2;
				if (selectedIndex == Integer.MIN_VALUE && position < halfWidget) {
					// A cursor flicker is caused by the request focus because the widgets than gain and lost the focus
					// are repainted. It can be solved by a management of the z-index (the cursor is always repaint
					// above the other ones).
					widget.requestFocus();

					if (position < currentSize) {
						// Above this item.
						selectedIndex = index - 1;
						int distance = currentSize - position;
						percent = (float) distance / previousSize;
					} else {
						// Below this item.
						selectedIndex = index;
						int distance = position - currentSize;
						percent = 1f - (float) distance / widgetSize;
					}
				}
				previousSize = widgetSize;
				currentSize += widgetSize;
				index++;
			}
		}
		if (selectedIndex <= -1 || selectedIndex == widgetsCount - 1) {
			if (position < 0) {
				// Before the first.
				selectedIndex = 0;
				percent = 1f;
			} else {
				// After the last.
				selectedIndex = widgetsCount - 1;
				if (cyclic) {
					int distance = position - currentSize;
					percent = -(float) distance / previousSize;
				} else {
					percent = 1f;
				}
			}
		}
		setSelectedIndex(selectedIndex);
		if (cursor != null) {
			cursor.setSelectedItem(selectedIndex, percent);
		}

		this.position = relativePosition;
	}

}
